@@ -9,6 +9,7 @@
 
 import logging
 
+
 class NullHandler(logging.Handler):
     def emit(self, record):
         pass
@@ -17,7 +18,6 @@ logging.getLogger("katcp").addHandler(NullHandler())
 
 del logging, NullHandler
 
-
 from .core import Message, KatcpSyntaxError, MessageParser, \
                   DeviceMetaclass, ExcepthookThread, FailReply, \
                   AsyncReply, KatcpDeviceError, KatcpClientError, \
@@ -27,6 +27,9 @@ from .server import DeviceServerBase, DeviceServer, DeviceLogger
 
 from .client import DeviceClient, BlockingClient, CallbackClient
 
-from .sensortree import GenericSensorTree, BooleanSensorTree, AggregateSensorTree
+from .sensortree import GenericSensorTree, BooleanSensorTree, \
+                        AggregateSensorTree
 
 from .version import VERSION, VERSION_STR
+
+__version__ = VERSION_STR
