@@ -3,6 +3,7 @@
 # vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
 # Copyright 2009 SKA South Africa (http://ska.ac.za/)
 # BSD license - see COPYING for details
+from __future__ import with_statement
 
 """Clients for the KAT device control language.
    """
@@ -97,8 +98,8 @@ class DeviceClient(object):
         msg : Message object
             The message to send.
         """
-        # TODO: should probably implement this as a queue of sockets and messages to send.
-        #       and have the queue processed in the main loop
+        # TODO: should probably implement this as a queue of sockets and
+        #       messages to send and have the queue processed in the main loop
         data = str(msg) + "\n"
         datalen = len(data)
         totalsent = 0
@@ -118,7 +119,8 @@ class DeviceClient(object):
                 try:
                     sent = sock.send(data[totalsent:])
                 except socket.error, e:
-                    if len(e.args) == 2 and e.args[0] == errno.EAGAIN and sock is self._sock:
+                    if len(e.args) == 2 and e.args[0] == errno.EAGAIN and \
+                            sock is self._sock:
                         continue
                     else:
                         send_failed = True
@@ -157,9 +159,11 @@ class DeviceClient(object):
             self._connect_failures += 1
             if self._connect_failures % 5 == 0:
                 # warn on every fifth failure
-                self._logger.warn("Failed to connect to %r: %s" % (self._bindaddr, e))
+                self._logger.warn("Failed to connect to %r: %s" %
+                                  (self._bindaddr, e))
             else:
-                self._logger.debug("Failed to connect to %r: %s" % (self._bindaddr, e))
+                self._logger.debug("Failed to connect to %r: %s" %
+                                   (self._bindaddr, e))
             sock.close()
             sock = None
 
@@ -210,8 +214,7 @@ class DeviceClient(object):
                 except Exception:
                     e_type, e_value, trace = sys.exc_info()
                     reason = "\n".join(traceback.format_exception(
-                        e_type, e_value, trace, self._tb_limit
-                    ))
+                        e_type, e_value, trace, self._tb_limit))
                     self._logger.error("BAD COMMAND: %s" % (reason,))
                 else:
                     self.handle_message(msg)
@@ -256,8 +259,7 @@ class DeviceClient(object):
         except Exception:
             e_type, e_value, trace = sys.exc_info()
             reason = "\n".join(traceback.format_exception(
-                e_type, e_value, trace, self._tb_limit
-            ))
+                e_type, e_value, trace, self._tb_limit))
             self._logger.error("Inform %s FAIL: %s" % (msg.name, reason))
 
     def handle_reply(self, msg):
@@ -277,8 +279,7 @@ class DeviceClient(object):
         except Exception:
             e_type, e_value, trace = sys.exc_info()
             reason = "\n".join(traceback.format_exception(
-                e_type, e_value, trace, self._tb_limit
-            ))
+                e_type, e_value, trace, self._tb_limit))
             self._logger.error("Reply %s FAIL: %s" % (msg.name, reason))
 
     def handle_request(self, msg):
@@ -305,8 +306,7 @@ class DeviceClient(object):
         except Exception:
             e_type, e_value, trace = sys.exc_info()
             reason = "\n".join(traceback.format_exception(
-                e_type, e_value, trace, self._tb_limit
-            ))
+                e_type, e_value, trace, self._tb_limit))
             self._logger.error("Request %s FAIL: %s" % (msg.name, reason))
 
     def unhandled_inform(self, msg):
@@ -341,8 +341,9 @@ class DeviceClient(object):
 
     def run(self):
         """Process reply and inform messages from the server."""
-        self._logger.debug("Starting thread %s" % (threading.currentThread().getName()))
-        timeout = 0.5 # s
+        self._logger.debug("Starting thread %s" % (
+                threading.currentThread().getName()))
+        timeout = 0.5  # s
 
         # save globals so that the thread can run cleanly
         # even while Python is setting module globals to
@@ -354,7 +355,8 @@ class DeviceClient(object):
         if not self._auto_reconnect:
             self._connect()
             if not self.is_connected():
-                raise KatcpClientError("Failed to connect to %r" % (self._bindaddr,))
+                raise KatcpClientError("Failed to connect to %r" %
+                                       (self._bindaddr,))
 
         self._running.set()
         while self._running.isSet():
@@ -364,9 +366,8 @@ class DeviceClient(object):
             sock = self._sock
             if sock is not None:
                 try:
-                    readers, _writers, errors = _select(
-                        [sock], [], [sock], timeout
-                    )
+                    readers, _writers, errors = _select([sock], [], [sock],
+                                                        timeout)
                 except Exception, e:
                     # catch Exception because class of exception thrown
                     # various drastically between Mac and Linux
@@ -400,7 +401,8 @@ class DeviceClient(object):
                         _sleep(timeout)
 
         self._disconnect()
-        self._logger.debug("Stopping thread %s" % (threading.currentThread().getName()))
+        self._logger.debug("Stopping thread %s" % (
+                threading.currentThread().getName()))
 
     def start(self, timeout=None, daemon=None, excepthook=None):
         """Start the client in a new thread.
@@ -545,7 +547,8 @@ class BlockingClient(DeviceClient):
     def __init__(self, host, port, tb_limit=20, timeout=5.0, logger=log,
                  auto_reconnect=True):
         super(BlockingClient, self).__init__(host, port, tb_limit=tb_limit,
-            logger=logger,auto_reconnect=auto_reconnect)
+                                             logger=logger,
+                                             auto_reconnect=auto_reconnect)
         self._request_timeout = timeout
 
         self._request_end = threading.Event()
@@ -717,7 +720,8 @@ class CallbackClient(DeviceClient):
     def __init__(self, host, port, tb_limit=20, timeout=5.0, logger=log,
                  auto_reconnect=True, use_ids=False):
         super(CallbackClient, self).__init__(host, port, tb_limit=tb_limit,
-            logger=logger,auto_reconnect=auto_reconnect)
+                                             logger=logger,
+                                             auto_reconnect=auto_reconnect)
 
         self._request_timeout = timeout
         self._use_ids = use_ids
@@ -730,20 +734,23 @@ class CallbackClient(DeviceClient):
         self._async_lock = threading.Lock()
 
         # pending requests
-        # msg_id -> (request_name, reply_cb, inform_cb, user_data, timer) callback tuples
+        # msg_id -> (request_name, reply_cb, inform_cb, user_data, timer)
+        #           callback tuples
         self._async_queue = {}
 
         # stack mapping request names to a stack of message ids
         # msg_name -> [ list of msg_ids ]
         self._async_id_stack = {}
 
-    def _push_async_request(self, msg_id, request_name, reply_cb, inform_cb, user_data, timer):
+    def _push_async_request(self, msg_id, request_name, reply_cb, inform_cb,
+                            user_data, timer):
         """Store the callbacks for a request we've sent so we
            can forward any replies and informs to them.
            """
         self._async_lock.acquire()
         try:
-            self._async_queue[msg_id] = (request_name, reply_cb, inform_cb, user_data, timer)
+            self._async_queue[msg_id] = (request_name, reply_cb, inform_cb,
+                                         user_data, timer)
             if request_name in self._async_id_stack:
                 self._async_id_stack[request_name].append(msg_id)
             else:
@@ -805,7 +812,8 @@ class CallbackClient(DeviceClient):
         finally:
             self._msg_id_lock.release()
 
-    def request(self, msg, reply_cb=None, inform_cb=None, user_data=None, timeout=None):
+    def request(self, msg, reply_cb=None, inform_cb=None, user_data=None,
+                timeout=None):
         """Send a request messsage.
 
         Parameters
@@ -825,16 +833,28 @@ class CallbackClient(DeviceClient):
             How long to wait for a reply. The default is the
             the timeout set when creating the CallbackClient.
         """
+        is_antenna = False
+        try:
+            if self.name == 'antenna':
+                is_antenna = True
+        except AttributeError:
+            pass
+
         if timeout is None:
             timeout = self._request_timeout
 
         msg_id = self._next_id()
-        timer = threading.Timer(timeout, self._handle_timeout, (msg_id,))
+        if timeout is None: # deal with 'no timeout', i.e. None
+            timer = None
+        else:
+            timer = threading.Timer(timeout, self._handle_timeout, (msg_id,))
 
-        self._push_async_request(msg_id, msg.name, reply_cb, inform_cb, user_data, timer)
+        self._push_async_request(msg_id, msg.name, reply_cb, inform_cb,
+                                 user_data, timer)
         if self._use_ids:
             msg.mid = msg_id
-        timer.start()
+        if timer:
+            timer.start()
         try:
             super(CallbackClient, self).request(msg)
         except KatcpClientError, e:
@@ -861,6 +881,12 @@ class CallbackClient(DeviceClient):
         informs : list of Message objects
             A list of the inform messages received.
         """
+        is_antenna = False
+        try:
+            if self.name == 'antenna':
+                is_antenna = True
+        except AttributeError:
+            pass
         if timeout is None:
             timeout = self._request_timeout
 
@@ -875,8 +901,21 @@ class CallbackClient(DeviceClient):
         def inform_cb(msg):
             informs.append(msg)
 
-        self.request(msg, reply_cb=reply_cb, inform_cb=inform_cb, timeout=timeout)
-        done.wait()
+        self.request(msg, reply_cb=reply_cb, inform_cb=inform_cb,
+                     timeout=timeout)
+        ## We wait on the done event that should be set by the reply
+        # handler callback. If this event does not occur within the
+        # timeout it means something unexpected went wrong. We give it
+        # an extra 5 seconds to deal with (unlikely?) slowness in the
+        # rest of the code
+        extra_wait = 5
+        wait_timeout = timeout
+        if not wait_timeout is None:
+            wait_timeout = wait_timeout + extra_wait
+        done.wait(timeout=wait_timeout)
+        if not done.isSet():
+            raise RuntimeError('Unexpected error: Async request handler did '
+                               'not call reply handler within timeout period')
         reply = replies[0]
 
         return reply, informs
@@ -896,11 +935,13 @@ class CallbackClient(DeviceClient):
         # inform_cb was passed to the request method.
         if self._use_ids:
             if msg.mid is not None:
-                _msg_name, _reply_cb, inform_cb, user_data, _timer = self._peek_async_request(msg.mid, None)
+                _msg_name, _reply_cb, inform_cb, user_data, _timer = \
+                    self._peek_async_request(msg.mid, None)
             else:
                 inform_cb, user_data = None, None
         else:
-            _msg_name, _reply_cb, inform_cb, user_data, _timer = self._peek_async_request(None, msg.name)
+            _msg_name, _reply_cb, inform_cb, user_data, _timer = \
+                self._peek_async_request(None, msg.name)
 
         if inform_cb is None:
             inform_cb = super(CallbackClient, self).handle_inform
@@ -915,9 +956,33 @@ class CallbackClient(DeviceClient):
         except Exception:
             e_type, e_value, trace = sys.exc_info()
             reason = "\n".join(traceback.format_exception(
-                e_type, e_value, trace, self._tb_limit
-            ))
-            self._logger.error("Callback inform %s FAIL: %s" % (msg.name, reason))
+                e_type, e_value, trace, self._tb_limit))
+            self._logger.error("Callback inform %s FAIL: %s" %
+                               (msg.name, reason))
+
+    def _do_fail_callback(self, reason, msg_name, reply_cb, inform_cb, user_data, timer):
+        """Do callback for a failed request"""
+        # this may also result in reply_cb being None if no
+        # reply_cb was passed to the request method
+
+        if reply_cb is None:
+            # this happens if no reply_cb was passed in to the request or
+            return
+
+        reason_msg = Message.reply(msg_name, "fail", reason)
+
+        try:
+            if user_data is None:
+                reply_cb(reason_msg)
+            else:
+                reply_cb(reason_msg, *user_data)
+        except Exception:
+            e_type, e_value, trace = sys.exc_info()
+            exc_reason = "\n".join(traceback.format_exception(
+                e_type, e_value, trace, self._tb_limit))
+            self._logger.error("Callback reply during failure %s, %s FAIL: %s" %
+                               (reason, msg_name, exc_reason))
+
 
     def _handle_timeout(self, msg_id):
         """Handle a timed out callback request.
@@ -927,27 +992,17 @@ class CallbackClient(DeviceClient):
         msg_id : uuid.UUID for message
             The name of the reply which was expected.
         """
-        # this may also result in reply_cb being None if no
-        # reply_cb was passed to the request method
-        msg_name, reply_cb, _inform_cb, user_data, timer = self._pop_async_request(msg_id, None)
-
-        if reply_cb is None:
-            # this happens if no reply_cb was passed in to the request or
+        msg_name, reply_cb, inform_cb, user_data, timer  = \
+            self._pop_async_request(msg_id, None)
+        # We may have been racing with the actual reply handler if the reply
+        # arrived close to the timeout expiry, which means the
+        # self._pop_async_request() call gave us None's. In this case, just bail
+        if timer is None:
             return
 
-        timeout_msg = Message.reply(msg_name, "fail", "Timed out after %f seconds" % timer.interval)
-
-        try:
-            if user_data is None:
-                reply_cb(timeout_msg)
-            else:
-                reply_cb(timeout_msg, *user_data)
-        except Exception:
-            e_type, e_value, trace = sys.exc_info()
-            reason = "\n".join(traceback.format_exception(
-                e_type, e_value, trace, self._tb_limit
-            ))
-            self._logger.error("Callback reply during timeout %s FAIL: %s" % (msg_name, reason))
+        reason = "Timed out after %f seconds" % timer.interval
+        self._do_fail_callback(
+            reason, msg_name, reply_cb, inform_cb, user_data, timer)
 
     def handle_reply(self, msg):
         """Handle a reply message related to the current request.
@@ -964,11 +1019,13 @@ class CallbackClient(DeviceClient):
         # reply_cb was passed to the request method
         if self._use_ids:
             if msg.mid is not None:
-                _msg_name, reply_cb, _inform_cb, user_data, timer = self._pop_async_request(msg.mid, None)
+                _msg_name, reply_cb, _inform_cb, user_data, timer = \
+                    self._pop_async_request(msg.mid, None)
             else:
                 reply_cb, user_data, timer = None, None, None
         else:
-            _msg_name, reply_cb, _inform_cb, user_data, timer = self._pop_async_request(None, msg.name)
+            _msg_name, reply_cb, _inform_cb, user_data, timer = \
+                self._pop_async_request(None, msg.name)
 
         if timer is not None:
             timer.cancel()
@@ -986,6 +1043,24 @@ class CallbackClient(DeviceClient):
         except Exception:
             e_type, e_value, trace = sys.exc_info()
             reason = "\n".join(traceback.format_exception(
-                e_type, e_value, trace, self._tb_limit
-            ))
-            self._logger.error("Callback reply %s FAIL: %s" % (msg.name, reason))
+                e_type, e_value, trace, self._tb_limit))
+            self._logger.error("Callback reply %s FAIL: %s" %
+                               (msg.name, reason))
+
+    def stop(self, *args, **kwargs):
+        super(CallbackClient, self).stop(*args, **kwargs)
+        # Stop all async timeout handlers
+        with self._async_lock:
+            for request_data in self._async_queue.values():
+                timer = request_data[-1]   # Last one should be timeout timer
+                if timer is not None:
+                    timer.cancel()
+                self._do_fail_callback('Client stopped before reply was received',
+                                       *request_data)
+
+    def join(self, timeout=None):
+        with self._async_lock:
+            for (_, _, _, _, timer) in self._async_queue.values():
+                if timer is not None:
+                    timer.join(timeout=timeout)
+        super(CallbackClient, self).join(timeout=timeout)
